=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-03-06 07:11:09 +0000
+++ bzrlib/branch.py	2006-03-07 09:03:44 +0000
@@ -207,7 +207,10 @@
             raise Exception("can't fetch from a branch to itself %s, %s" % 
                             (self.base, to_branch.base))
         if pb is None:
-            pb = bzrlib.ui.ui_factory.progress_bar()
+            nested_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+            pb = nested_pb
+        else:
+            nested_pb = None
 
         from_branch.lock_read()
         try:
@@ -221,8 +224,10 @@
                     last_revision = NULL_REVISION
             return self.repository.fetch(from_branch.repository,
                                          revision_id=last_revision,
-                                         pb=pb)
+                                         pb=nested_pb)
         finally:
+            if nested_pb is not None:
+                nested_pb.finished()
             from_branch.unlock()
 
     def get_bound_location(self):

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2006-03-03 09:41:00 +0000
+++ bzrlib/check.py	2006-03-07 09:03:44 +0000
@@ -55,7 +55,7 @@
 
     def check(self):
         self.branch.lock_read()
-        self.progress = bzrlib.ui.ui_factory.progress_bar()
+        self.progress = bzrlib.ui.ui_factory.nested_progress_bar()
         try:
             self.progress.update('retrieving inventory', 0, 0)
             # do not put in init, as it should be done with progess,
@@ -75,7 +75,7 @@
                 revno += 1
                 self.check_one_rev(rev_id)
         finally:
-            self.progress.clear()
+            self.progress.finished()
             self.branch.unlock()
 
     def plan_revisions(self):

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2006-02-28 01:13:01 +0000
+++ bzrlib/fetch.py	2006-03-07 09:03:44 +0000
@@ -94,15 +94,19 @@
         # must not mutate self._last_revision as its potentially a shared instance
         self._last_revision = last_revision
         if pb is None:
-            self.pb = bzrlib.ui.ui_factory.progress_bar()
+            self.pb = bzrlib.ui.ui_factory.nested_progress_bar()
+            self.nested_pb = self.pb
         else:
             self.pb = pb
+            self.nested_pb = None
         self.from_repository.lock_read()
         try:
             self.to_repository.lock_write()
             try:
                 self.__fetch()
             finally:
+                if self.nested_pb is not None:
+                    self.nested_pb.finished()
                 self.to_repository.unlock()
         finally:
             self.from_repository.unlock()

=== modified file 'bzrlib/missing.py'
--- bzrlib/missing.py	2006-02-16 21:29:34 +0000
+++ bzrlib/missing.py	2006-03-07 09:03:44 +0000
@@ -27,7 +27,7 @@
 
 
 def find_unmerged(local_branch, remote_branch):
-    progress = ui.ui_factory.progress_bar()
+    progress = ui.ui_factory.nested_progress_bar()
     local_branch.lock_read()
     try:
         remote_branch.lock_read()
@@ -61,7 +61,7 @@
             remote_branch.unlock()
     finally:
         local_branch.unlock()
-        progress.clear()
+        progress.finished()
     return (local_extra, remote_extra)
 
 def _shortcut(local_rev_history, remote_rev_history):

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2006-03-07 07:57:33 +0000
+++ bzrlib/progress.py	2006-03-07 09:03:44 +0000
@@ -75,7 +75,8 @@
                  show_eta=True,
                  show_bar=True,
                  show_count=True,
-                 to_messages_file=sys.stdout):
+                 to_messages_file=sys.stdout,
+                 klass=None):
         """Setup the stack with the parameters the progress bars should have."""
         self._to_file = to_file
         self._show_pct = show_pct
@@ -85,18 +86,19 @@
         self._show_count = show_count
         self._to_messages_file = to_messages_file
         self._stack = []
+        self._klass = klass or TTYProgressBar
 
     def get_nested(self):
         """Return a nested progress bar."""
         # initial implementation - return a new bar each time.
-        new_bar = TTYProgressBar(to_file=self._to_file,
-                                 show_pct=self._show_pct,
-                                 show_spinner=self._show_spinner,
-                                 show_eta=self._show_eta,
-                                 show_bar=self._show_bar,
-                                 show_count=self._show_count,
-                                 to_messages_file=self._to_messages_file,
-                                 _stack=self)
+        new_bar = self._klass(to_file=self._to_file,
+                              show_pct=self._show_pct,
+                              show_spinner=self._show_spinner,
+                              show_eta=self._show_eta,
+                              show_bar=self._show_bar,
+                              show_count=self._show_count,
+                              to_messages_file=self._to_messages_file,
+                              _stack=self)
         self._stack.append(new_bar)
         return new_bar
 
@@ -134,7 +136,7 @@
     def finished(self):
         """Return this bar to its progress stack."""
         self.clear()
-        assert self._stack
+        assert self._stack is not None
         self._stack.return_pb(self)
 
     def note(self, fmt_string, *args, **kwargs):
@@ -163,6 +165,7 @@
 
 
 class DotsProgressBar(_BaseProgressBar):
+
     def __init__(self, **kwargs):
         _BaseProgressBar.__init__(self, **kwargs)
         self.last_msg = None

=== modified file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	2006-02-28 01:13:01 +0000
+++ bzrlib/reconcile.py	2006-03-07 09:03:44 +0000
@@ -57,7 +57,14 @@
         garbage_inventories: The number of inventory objects without revisions
                              that were garbage collected.
         """
-        self.pb = ui.ui_factory.progress_bar()
+        self.pb = ui.ui_factory.nested_progress_bar()
+        try:
+            self._reconcile()
+        finally:
+            self.pb.finished()
+
+    def _reconcile(self):
+        """Helper function for performing reconciliation."""
         self.repo = self.bzrdir.find_repository()
         self.pb.note('Reconciling repository %s',
                      self.repo.bzrdir.root_transport.base)
@@ -86,10 +93,13 @@
         garbage_inventories: The number of inventory objects without revisions
                              that were garbage collected.
         """
-        self.pb = ui.ui_factory.progress_bar()
         self.repo.lock_write()
         try:
-            self._reweave_inventory()
+            self.pb = ui.ui_factory.nested_progress_bar()
+            try:
+                self._reweave_inventory()
+            finally:
+                self.pb.finished()
         finally:
             self.repo.unlock()
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-03-06 07:44:17 +0000
+++ bzrlib/repository.py	2006-03-07 09:03:44 +0000
@@ -1289,14 +1289,17 @@
                 pass
             # FIXME do not peek!
             if self.source.control_files._transport.listable():
-                pb = bzrlib.ui.ui_factory.progress_bar()
-                copy_all(self.source.weave_store,
-                    self.target.weave_store, pb=pb)
-                pb.update('copying inventory', 0, 1)
-                self.target.control_weaves.copy_multi(
-                    self.source.control_weaves, ['inventory'])
-                copy_all(self.source.revision_store,
-                    self.target.revision_store, pb=pb)
+                pb = bzrlib.ui.ui_factory.nested_progress_bar()
+                try:
+                    copy_all(self.source.weave_store,
+                        self.target.weave_store, pb=pb)
+                    pb.update('copying inventory', 0, 1)
+                    self.target.control_weaves.copy_multi(
+                        self.source.control_weaves, ['inventory'])
+                    copy_all(self.source.revision_store,
+                        self.target.revision_store, pb=pb)
+                finally:
+                    pb.finished()
             else:
                 self.target.fetch(self.source, revision_id=revision_id)
 

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-03-06 07:11:09 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-03-07 09:03:44 +0000
@@ -92,12 +92,18 @@
     def clear(self):
         """See progress.ProgressBar.clear()."""
 
+    def finished(self):
+        """See progress.ProgressBar.finished()."""
+
     def note(self, fmt_string, *args, **kwargs):
         """See progress.ProgressBar.note()."""
         print fmt_string % args
 
     def progress_bar(self):
         return self
-        
+    
+    def nested_progress_bar(self):
+        return self
+
     def update(self, message, count=None, total=None):
         """See progress.ProgressBar.update()."""

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-03-03 13:22:46 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-03-07 09:03:44 +0000
@@ -1071,11 +1071,12 @@
             # as downgrades may not be available
             old_format = bzrdir.BzrDirFormat.get_default_format()
             bzrdir.BzrDirFormat.set_default_format(dir._format)
+            pb = ui.ui_factory.nested_progress_bar()
             try:
-                dir._format.get_converter(None).convert(dir,
-                    ui.ui_factory.progress_bar())
+                dir._format.get_converter(None).convert(dir, pb)
             finally:
                 bzrdir.BzrDirFormat.set_default_format(old_format)
+                pb.finished()
             # and it should pass 'check' now.
             check(bzrdir.BzrDir.open(self.get_url('.')).open_branch(), False)
 

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2006-03-06 04:43:59 +0000
+++ bzrlib/tests/test_repository.py	2006-03-07 09:03:44 +0000
@@ -422,10 +422,12 @@
         repo_dir = bzrdir.BzrDirMetaFormat1().initialize('repository')
         repo = repository.RepositoryFormat7().initialize(repo_dir)
         target_format = repository.RepositoryFormatKnit1()
-        pb = bzrlib.ui.ui_factory.progress_bar()
-
         converter = repository.CopyConverter(target_format)
-        converter.convert(repo, pb)
+        pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        try:
+            converter.convert(repo, pb)
+        finally:
+            pb.finished()
         repo = repo_dir.open_repository()
         self.assertTrue(isinstance(target_format, repo._format.__class__))
 

=== modified file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	2006-03-07 07:57:33 +0000
+++ bzrlib/tests/test_ui.py	2006-03-07 09:03:44 +0000
@@ -33,17 +33,21 @@
     def test_silent_factory(self):
 
         ui = SilentUIFactory()
-        pb = ui.progress_bar()
-        # TODO: Test that there is no output from SilentUIFactory
-
-        self.assertEquals(ui.get_password(), None)
-        self.assertEquals(ui.get_password(u'Hello There \u1234 %(user)s',
-                                          user=u'some\u1234')
-                         , None)
+        pb = ui.nested_progress_bar()
+        try:
+            # TODO: Test that there is no output from SilentUIFactory
+    
+            self.assertEquals(ui.get_password(), None)
+            self.assertEquals(ui.get_password(u'Hello There \u1234 %(user)s',
+                                              user=u'some\u1234')
+                             , None)
+        finally:
+            pb.finished()
 
     def test_text_factory(self):
         ui = TextUIFactory()
-        pb = ui.progress_bar()
+        pb = ui.nested_progress_bar()
+        pb.finished()
         # TODO: Test the output from TextUIFactory, perhaps by overriding sys.stdout
 
         # Unfortunately we can't actually test the ui.get_password() because 

=== modified file 'bzrlib/ui/__init__.py'
--- bzrlib/ui/__init__.py	2006-03-07 08:06:38 +0000
+++ bzrlib/ui/__init__.py	2006-03-07 09:03:44 +0000
@@ -40,6 +40,10 @@
     layer different applications can choose the style of UI.
     """
 
+    def __init__(self):
+        super(UIFactory, self).__init__()
+        self._progress_bar_stack = None
+
     @deprecated_method(zero_eight)
     def progress_bar(self):
         """See UIFactory.nested_progress_bar()."""
@@ -81,7 +85,10 @@
         return None
 
     def nested_progress_bar(self):
-        return bzrlib.progress.DummyProgress()
+        if self._progress_bar_stack is None:
+            self._progress_bar_stack = bzrlib.progress.ProgressBarStack(
+                klass=bzrlib.progress.DummyProgress)
+        return self._progress_bar_stack.get_nested()
 
 
 ui_factory = SilentUIFactory()

=== modified file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	2006-03-07 08:06:38 +0000
+++ bzrlib/ui/text.py	2006-03-07 09:03:44 +0000
@@ -29,10 +29,6 @@
 
 class TextUIFactory(UIFactory):
 
-    def __init__(self):
-        super(TextUIFactory, self).__init__()
-        self._progress_bar_stack = None
-
     @deprecated_method(zero_eight)
     def progress_bar(self):
         """See UIFactory.nested_progress_bar()."""

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2006-02-21 01:05:33 +0000
+++ bzrlib/upgrade.py	2006-03-07 09:03:44 +0000
@@ -33,10 +33,13 @@
         if self.bzrdir.root_transport.is_readonly():
             raise errors.UpgradeReadonly
         self.transport = self.bzrdir.root_transport
-        self.convert()
+        self.pb = ui.ui_factory.nested_progress_bar()
+        try:
+            self.convert()
+        finally:
+            self.pb.finished()
 
     def convert(self):
-        self.pb = ui.ui_factory.progress_bar()
         branch = self.bzrdir.open_branch()
         if branch.bzrdir.root_transport.base != self.bzrdir.root_transport.base:
             self.pb.note("This is a checkout. The branch (%s) needs to be "

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-03-06 22:11:12 +0000
+++ bzrlib/workingtree.py	2006-03-07 09:03:44 +0000
@@ -855,11 +855,15 @@
                 else:
                     other_revision = None
                 repository = self.branch.repository
-                merge_inner(self.branch,
-                            self.branch.basis_tree(),
-                            basis_tree, 
-                            this_tree=self, 
-                            pb=bzrlib.ui.ui_factory.progress_bar())
+                pb = bzrlib.ui.ui_factory.nested_progress_bar()
+                try:
+                    merge_inner(self.branch,
+                                self.branch.basis_tree(),
+                                basis_tree, 
+                                this_tree=self, 
+                                pb=pb)
+                finally:
+                    pb.finished()
                 self.set_last_revision(self.branch.last_revision())
             return count
         finally:

